/*	File			DB_Arch_AppClient_for_SingleAppServer_DB1andDB2.java
	Purpose			DB Client component for illustration of aspects of connectivity, coupling, architecture and transparency
	Author			Richard Anthony	(ar26@gre.ac.uk)
	Date			March 2015
*/
package DB_Arch_AppClient_for_SingleAppServer_DB1andDB2;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.*;
import java.util.Arrays;
import java.util.Timer;
import java.util.TimerTask;
import javax.swing.*;
    
public class DB_Arch_AppClient_for_SingleAppServer_DB1andDB2 extends javax.swing.JFrame implements ActionListener
{
    private static int DB_TYPE1_PORT = 8002;
    private static int DB_TYPE2_PORT = 8003;
    private static int App_TYPE1_PORT = 8004;
    private static int App_TYPE2_PORT = 8005;
    private static int DATABASE_SIZE = 3;
    private static int MAX_CUST_NAME_LEN = 20;
    
    private enum MessageType {REQUEST_RECORD, SEND_RECORD, CLIENT_DISCONNECT};

    // To implement the equivallent of a C/C++/C# structure, in Java a class is used
    final class Application_Message_PDU
    {
        MessageType eMessageType;   // Size 4   Offset 0
        int iCust_ID;               // Size 4   Offset 4
        Boolean bCustNameValid;     // Size 1   Offset 8
        byte[] cCustName;           // Size 21  Offset 9 - 29
        Boolean bCustBalanceValid;  // Size 1   Offset 30
        // Single byte of padding exists in the byte arraay at this point, to align the following int on a 4-byte boundary
        int iCustBalance;           // Size 32  Offset 32
    };

    private static int Application_Message_Size = 36;

    Socket m_ConnectSock_App;
    InputStream m_InStream_App;
    OutputStream m_OutStream_App;
    InetAddress m_Connect_App_Address;
    Boolean m_bConnected_App;
    Timer m_Timer;

    public DB_Arch_AppClient_for_SingleAppServer_DB1andDB2()
    {
        initComponents();
        m_bConnected_App = false;
        InitialiseControls();
        GetLocalIPAddress();
    }

    private void InitialiseControls()    
    {    
        jTextField_Port.setEnabled(false);
        jTextField_Port.setText(Integer.toString(App_TYPE1_PORT));
        jTextField_IPAddress.setEnabled(true);
        jButton_Connect.setEnabled(true);
        jTextField_CustomerID.setEnabled(false);;
        jButton_RequestData.setEnabled(false);
        jTextField_CustomerName.setEnabled(false);
        jTextField_CustomerBalance.setEnabled(false);
        jButton_Done.setEnabled(true);        
    }

    void GetLocalIPAddress()
    {   
        String sHostAddress = "";
        try
        {
            InetAddress LocalAddress = InetAddress.getLocalHost();
            sHostAddress = LocalAddress.toString(); // Expected format: "Hostname/192.168.100.5"
            int iStartOfDottedIPSubstring = sHostAddress.indexOf('/');
            if(-1 == iStartOfDottedIPSubstring)
            {   // Align to the actual start of the dotted IP address part of the string
                iStartOfDottedIPSubstring = 0; // '/' not found
            }
            else
            {
                iStartOfDottedIPSubstring++; // Skip the '/' charachter
            }
            String sDottedIPAddress = sHostAddress.substring(iStartOfDottedIPSubstring);
            jTextField_IPAddress.setText(sDottedIPAddress);
        }
        catch (Exception Ex)
        {
            JOptionPane.showMessageDialog(null,Ex.toString(),"DB Arch AppClient",JOptionPane.PLAIN_MESSAGE);
            CloseAndExit();
        }
    }

    private void Start_Receive_Timer()
    {
        m_Timer = new Timer();
        m_Timer.scheduleAtFixedRate(
                new OnTimedEvent_Receive_Timer(), 100, 100); // Short interval to allow UI responsiveness)
    }

    private class OnTimedEvent_Receive_Timer extends TimerTask
    {
        public void run()
        {
            DoReceive();
        }
    }
         
    void DoReceive()
    {		
        byte[] bArray = new byte[Application_Message_Size];
        int iReceiveByteCount = 0;
        if (true == m_bConnected_App)
        {
            iReceiveByteCount = 0;
            try
            {
                iReceiveByteCount = m_InStream_App.read(bArray);
                if(Application_Message_Size <= iReceiveByteCount)
                {
                    Application_Message_PDU Message = DeSerialize_ApplicationMessage(bArray);
                    switch (Message.eMessageType)
                    {
                        case SEND_RECORD:
                            DisplayData(Message);
                            break;
                    }
                }
            }
            catch (SocketTimeoutException Ex)
            {
            }
            catch (Exception Ex)
            {
            }
        }
    }

    Application_Message_PDU DeSerialize_ApplicationMessage(byte[] bRecdByteArray)
    {   // Converts a byte array (from Receive) to an Application_Message_PDU structure
        Application_Message_PDU result_Message = new Application_Message_PDU();
        result_Message.eMessageType = MessageType.values()[bRecdByteArray[0]]; // Convert byte value to enumerated MessageType
        byte[] bInt = new byte[4]; 
        System.arraycopy(bRecdByteArray/*src*/, 4, bInt/*dest*/, 0, 4/*count*/); 
        result_Message.iCust_ID = ByteArraySize4_ToInt(bInt);
        result_Message.bCustNameValid = true;
        if (0 == bRecdByteArray[8])
        {
            result_Message.bCustNameValid = false;
        }
        result_Message.cCustName = new byte[MAX_CUST_NAME_LEN+1];
        System.arraycopy(bRecdByteArray/*src*/, 9, result_Message.cCustName/*dest*/, 0, MAX_CUST_NAME_LEN/*count*/); 
        result_Message.bCustBalanceValid = true;
        if (0 == bRecdByteArray[30])
        {
            result_Message.bCustBalanceValid = false;
        }
        System.arraycopy(bRecdByteArray/*src*/, 32, bInt/*dest*/, 0, 4/*count*/); 
        result_Message.iCustBalance = ByteArraySize4_ToInt(bInt);
        return result_Message;
    }

    private byte[] Serialize_Application_Message_PDU(Application_Message_PDU Message)
    {   // Convert an Application_Message_PDU structure into a byte array
        byte[] bByteArray = new byte[Application_Message_Size]; // Create a byte array to hold the serialised form of the message PDU
        bByteArray[0] = (byte)Message.eMessageType.ordinal();
        byte[] bInt = IntToByteArray(Message.iCust_ID);
        System.arraycopy(bInt/*src*/, 0, bByteArray/*dest*/, 4, 4/*count*/);
        bByteArray[8] = 0; // false (bCustNameValid)
        if(true == Message.bCustNameValid)
        {
            bByteArray[8] = 1; // true (bCustNameValid)
        }
        if(null != Message.cCustName)
        {
            System.arraycopy(Message.cCustName/*src*/, 0, bByteArray/*dest*/, 9, Message.cCustName.length/*count*/);
        }
        bByteArray[30] = 0; // false (bCustBalanceValid)
        if (true == Message.bCustBalanceValid)
        {
            bByteArray[30] = 1; // true (bCustBalanceValid)
        }
        bInt = IntToByteArray(Message.iCustBalance);
        System.arraycopy(bInt/*src*/, 0, bByteArray/*dest*/, 32, 4/*count*/);
        return bByteArray;
    }

    int GetCustIDFromControl()
    {
        int iCustID = 0;
        if ("" == jTextField_CustomerID.getText())
        {
            JOptionPane.showMessageDialog(null,
                "Customer ID must be between 101 and 103", "Invalid Customer ID",
                JOptionPane.PLAIN_MESSAGE);
            return -1;
        }
        try
        {
            iCustID = Integer.parseInt(jTextField_CustomerID.getText());
            if (iCustID < 101 || iCustID > 103)
            {
            JOptionPane.showMessageDialog(null,
                "Customer ID must be between 101 and 103", "Invalid Customer ID",
                JOptionPane.PLAIN_MESSAGE);
                return -1;
            }
        }
        catch (Exception Ex)
        {
            JOptionPane.showMessageDialog(null, Ex.toString(), "Invalid Customer ID",
                JOptionPane.PLAIN_MESSAGE);
            return -1;
        }
        return iCustID;
    }    

    void RequestData()
    {
        int iCustID = GetCustIDFromControl();
        if (-1 == iCustID)
        {
            return;
        }
        jTextField_CustomerName.setText("");
        jTextField_CustomerBalance.setText("");
        Application_Message_PDU Message = new Application_Message_PDU();
        Message.eMessageType = MessageType.REQUEST_RECORD;
        Message.iCust_ID = iCustID;
        Message.bCustNameValid = false;
        Message.bCustBalanceValid = false;
        if(true == m_bConnected_App)
        {
            SendApplicationMessage(Message); // Send REQUEST_RECORD message to AppServer
        }	
    }

    void SendApplicationMessage(Application_Message_PDU Message)
    {
        byte[] bBuffer = Serialize_Application_Message_PDU(Message);
        try
        {
            m_OutStream_App.write(bBuffer);
        }
        catch (SocketTimeoutException Ex)
        {
        }
        catch (Exception ex)
        {
        }
    }

    void CreateAndConnectSocket()
    {
        if(true == GetConnectAddress_App())
        {
            try
            {
                m_ConnectSock_App = new Socket(m_Connect_App_Address, App_TYPE1_PORT); // Create Socket and connect to Server
                m_InStream_App = m_ConnectSock_App.getInputStream();
                m_OutStream_App = m_ConnectSock_App.getOutputStream();
                m_ConnectSock_App.setSoTimeout(0); // Timeout = 0  indicates infinite timeout (i.e blocking IO behaviour)
            }  
            catch (Exception Ex)  // Catch exceptions from the socket operations
            {
               JOptionPane.showMessageDialog(null,Ex.toString(),
                   "Is server address/port correct? Is server running?",
                   JOptionPane.PLAIN_MESSAGE);
               return;
            }
            m_bConnected_App = true;
            jTextField_CustomerID.setEnabled(true);
            jButton_RequestData.setEnabled(true);
            jTextField_IPAddress.setEnabled(false);
            jButton_Connect.setEnabled(false);
            Start_Receive_Timer();
        }
    }

    Boolean GetConnectAddress_App()
    {
        // Get the server address fron the user interface. Our own address is initially placed there to
        // reduce user interaction requirements (for example if both processes are on the same computer
        // no change to the address is required, or if both computers in the same Class C IP subnet
        // only the fourth address byte needs to be changed). The port number is also initialised on
        // the user interface to a default value, but must be read-in because the use may have changed it
        byte[] bArray = DottedIPAddessString_To_ByteArraySize4(jTextField_IPAddress.getText());
        if(0 == bArray[0])
        {
            JOptionPane.showMessageDialog(null,"Mal-formed dotted IP address (use format n.n.n.n)","DB Arch AppClient",JOptionPane.PLAIN_MESSAGE);
            return false;
        }
        try
        {
            m_Connect_App_Address = InetAddress.getByAddress(bArray);
        }
        catch (Exception Ex)
        {
            JOptionPane.showMessageDialog(null,Ex.toString(),"DB Arch AppClient",JOptionPane.PLAIN_MESSAGE);
            return false;
        }
        return true;
    }

    void DisplayData(Application_Message_PDU Message)
    {
        if(true == Message.bCustNameValid) // Guard needed because each server returns only a partial row of data at the application level
        {
            String sName = new String(Message.cCustName);
            sName = TruncateString(sName, sName.length());
            jTextField_CustomerName.setText(sName);
        }
        if(true == Message.bCustBalanceValid) // Guard needed because each server returns only a partial row of data at the application level
        {
            jTextField_CustomerBalance.setText(Integer.toString(Message.iCustBalance));
        }
    }

    void CloseSocket()
    {
        try
        {
            m_ConnectSock_App.close();
        }
        catch (Exception Ex)
        {   // Generic approach to dealing with situations such as socket not connected
        }
    }

    private void CloseAndExit()
    {   // Close socket and exit the application
        if(null != m_Timer)
        {
            m_Timer.cancel();
        }
        CloseSocket();
        System.exit(0);
    }
        
    private static byte[] IntToByteArray(int iVal)
    {
        byte[] bArray = new byte[4];
        bArray[0] = (byte)(iVal & 0x000000ff);
        bArray[1] = (byte)((iVal & 0x0000ff00) >> 8);
        bArray[2] = (byte)((iVal & 0x00ff0000) >> 16);
        bArray[3] = (byte)((iVal & 0xff000000) >> 24);
        return bArray;
    }

    private int ByteArraySize4_ToInt(byte[] bVal)
    {
        if(4 != bVal.length)
        {
            return -1;  // Signal invalid byte array length
        }
        int iResult = bVal[0] & 0xFF;
        iResult += ((bVal[1] & 0xFF) << 8);
        iResult += ((bVal[2] & 0xFF) << 16);
        iResult += ((bVal[3] & 0xFF) << 24);            
        return iResult;
    }
    
    private String ByteArraySize4_To_IPAddessString(byte[] bVal)
    {
        if(4 != bVal.length)
        {
            return "";  // Signal invalid byte array length
        }
        int iAddr1 = bVal[0] & 0xFF;
        int iAddr2 = bVal[1] & 0xFF;
        int iAddr3 = bVal[2] & 0xFF;
        int iAddr4 = bVal[3] & 0xFF;
        String sAddress = String.format("%d.%d.%d.%d", iAddr1, iAddr2, iAddr3, iAddr4);
        return sAddress;
    }

    private byte[] DottedIPAddessString_To_ByteArraySize4(String sDottedIPAddress)
    {
        byte[] bAddress = new byte[4]; // Each byte initialised as value 0
        int iLength = sDottedIPAddress.length();
        if(7 > iLength || 15 < iLength) // A well formed dotted IP address string must be in this length range
        {
            return bAddress; // All 0's signals mal-formed dotted IPaddress string
        }
        // sDottedIPAddress is a '.' separated, list of numeric values
        // There is no '.' after the last item in the list
        int iNextDotPos  = 0;
        String sCurrentValue = "";
        int iArrayIndex = 0;
        try
        {
            while ((iNextDotPos = sDottedIPAddress.indexOf('.')) > 0 && iNextDotPos < iLength) // Ignore chars beyond the valid string content
            {
                sCurrentValue = sDottedIPAddress.substring(0/*start*/, iNextDotPos/*length*/);
                bAddress[iArrayIndex++] = (byte)Integer.parseInt(sCurrentValue);
                sDottedIPAddress = sDottedIPAddress.substring(iNextDotPos + 1/*start*/);// Remove 'dealt with' name from head of list
                iLength -= (iNextDotPos + 1);                                 // Keep track of remaining list-string length
            }
            // Deal with residual substring (there is no comma after the last item in the list)
            bAddress[iArrayIndex++] = (byte)Integer.parseInt(sDottedIPAddress);
        }
        catch (Exception Ex)
        {   // Catch various string-format related exceptions (such as having more than three '.'s which
            // would cause the byte array index to go out of bounds)
            bAddress[0] = 0; // signal mal-formed dotted IPaddress string 
        }
        return bAddress;
    }

    private String TruncateString(String sStr, int iMaxLength)
    {
        try
        {
            // Truncate to null terminator if string contains null-termiation chars (happens when converting from a byte array)
            int iNullTerminatePosition = sStr.indexOf(0);
            if(-1 != iNullTerminatePosition)
            {
                sStr = sStr.substring(0, iNullTerminatePosition);
            }
            if(sStr.length()> iMaxLength)
            {   // Only use substring if the string is actually longer than the second parameter
                // (the length limit) otherwise an ArgumentOutOfRangeException is thrown
                sStr = sStr.substring(0, iMaxLength);
            }
            return sStr;
        }
        catch (Exception Ex)
        {
            return "";
        }
    }

    public static void main(String args[]) throws Exception
    {
        // Initialise the GUI libraries
        try 
        {
            javax.swing.UIManager.LookAndFeelInfo[] installedLookAndFeels=javax.swing.UIManager.getInstalledLookAndFeels();
            for (int idx=0; idx<installedLookAndFeels.length; idx++)
            {
                if ("Nimbus".equals(installedLookAndFeels[idx].getName())) {
                    javax.swing.UIManager.setLookAndFeel(installedLookAndFeels[idx].getClassName());
                    break;
                }
            }
        } 
        catch (Exception Ex)
        {
            System.exit(0);
        }

        // Create and display the GUI form
        java.awt.EventQueue.invokeAndWait(new Runnable()
        { 
            public void run() 
            { 
                new DB_Arch_AppClient_for_SingleAppServer_DB1andDB2().setVisible(true);
            }
        } );
    }    

    public void actionPerformed(ActionEvent e)
    {
        if(jButton_Done == e.getSource())
        {
            CloseAndExit();
        }
        if(jButton_Connect == e.getSource())
        {
            CreateAndConnectSocket();
        }
        if(jButton_RequestData == e.getSource())
        {
            RequestData();
        }
    }
    
    private JPanel jPanel_AppServer;
    private JLabel jLabel_Port;
    private JTextField jTextField_Port;
    private JLabel jLabel_IPAddress;
    private JTextField jTextField_IPAddress;
    private JButton jButton_Connect;
    private JPanel jPanel_ControlsAndData;
    private JLabel jLabel_CustomerID;
    private JTextField jTextField_CustomerID;
    private JButton jButton_RequestData;
    private JLabel jLabel_CustomerName;
    private JTextField jTextField_CustomerName;
    private JLabel jLabel_CustomerBalance;
    private JTextField jTextField_CustomerBalance;
    private JButton jButton_Done;

    private void initComponents()
    {
        jLabel_Port = new JLabel();
        jLabel_Port.setText("Port");
        jTextField_Port = new JTextField();
        jTextField_Port.setMaximumSize(new Dimension(50, 4));
        jTextField_Port.setHorizontalAlignment(JTextField.CENTER);
        jLabel_IPAddress = new JLabel();
        jLabel_IPAddress.setText("Server IP Address");
        jTextField_IPAddress = new JTextField();
        jTextField_IPAddress.setMaximumSize(new Dimension(100, 4));
        jTextField_IPAddress.setHorizontalAlignment(JTextField.CENTER);
        jButton_Connect = new JButton();
        jButton_Connect.setText("Connect");
        jButton_Connect.setMaximumSize(new Dimension(90, 6));
        jButton_Connect.addActionListener(this);
        jPanel_AppServer = new JPanel();
        jPanel_AppServer.setPreferredSize(new Dimension(260, 200));
        jPanel_AppServer.setBorder(BorderFactory.createTitledBorder("App Server connection configuration"));

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel_AppServer);
        jPanel_AppServer.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                .add(jPanel1Layout.createSequentialGroup()
                    .add(jLabel_Port)
                    .addContainerGap(10, 10)
                    .add(jTextField_Port))
                .add(jPanel1Layout.createSequentialGroup()
                    .addContainerGap(10, 10)
                    .add(jLabel_IPAddress)
                    .addContainerGap(10, 10)
                    .add(jTextField_IPAddress)                    
                    .addContainerGap(10, 10))
                .add(jPanel1Layout.createSequentialGroup()
                    .add(jButton_Connect)));
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                .add(jPanel1Layout.createSequentialGroup()
                    .add(jPanel1Layout.createParallelGroup()
                        .add(jLabel_Port)
                        .add(jTextField_Port))
                    .addContainerGap(10, 10)
                        .add(jPanel1Layout.createParallelGroup()
                        .add(jLabel_IPAddress)
                        .add(jTextField_IPAddress))            
                        .addContainerGap(10, 10)
                    .add(jPanel1Layout.createParallelGroup()
                        .add(jButton_Connect))));

        jLabel_CustomerID = new JLabel();
        jLabel_CustomerID.setText(" ID of Customer (use values 101, 102, 103) ");
        jTextField_CustomerID = new JTextField();
        jTextField_CustomerID.setMaximumSize(new Dimension(50, 4));
        jTextField_CustomerID.setHorizontalAlignment(JTextField.CENTER);
        jButton_RequestData = new JButton();
        jButton_RequestData.setText("Request data");
        jButton_RequestData.setMaximumSize(new Dimension(100, 4));
        jButton_RequestData.addActionListener(this);
        jLabel_CustomerName = new JLabel();
        jLabel_CustomerName.setText("Customer name");
        jTextField_CustomerName = new JTextField();
        jTextField_CustomerName.setMaximumSize(new Dimension(80, 4));
        jTextField_CustomerName.setHorizontalAlignment(JTextField.CENTER);
        jLabel_CustomerBalance = new JLabel();
        jLabel_CustomerBalance.setText("Customer account balance");
        jTextField_CustomerBalance = new JTextField();
        jTextField_CustomerBalance.setMaximumSize(new Dimension(60, 4));
        jTextField_CustomerBalance.setHorizontalAlignment(JTextField.CENTER);
        jButton_Done = new JButton();
        jButton_Done.setText("Done");
        jButton_Done.setMaximumSize(new Dimension(100, 4));
        jButton_Done.addActionListener(this);
        jPanel_ControlsAndData = new JPanel();
        jPanel_ControlsAndData.setPreferredSize(new Dimension(260, 200));
        jPanel_ControlsAndData.setBorder(BorderFactory.createTitledBorder(""));

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel_ControlsAndData);
        jPanel_ControlsAndData.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                    .add(jLabel_CustomerID)
                    .add(jPanel2Layout.createSequentialGroup()
                        .addContainerGap(10, 10)
                        .add(jTextField_CustomerID)
                        .addContainerGap(10, 10)
                        .add(jButton_RequestData)
                        .addContainerGap(10, 10))
                    .add(jPanel2Layout.createSequentialGroup()
                        .addContainerGap(10, 10)
                        .add(jLabel_CustomerName)
                        .addContainerGap(10, 10)
                        .add(jTextField_CustomerName)
                        .addContainerGap(10, 10))
                    .add(jPanel2Layout.createSequentialGroup()
                        .add(jLabel_CustomerBalance)
                        .addContainerGap(10, 10)
                        .add(jTextField_CustomerBalance))
                        .add(jPanel2Layout.createSequentialGroup()
                        .add(jButton_Done))));
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
            .add(jPanel2Layout.createSequentialGroup()
                .add(jLabel_CustomerID)
                .add(jPanel2Layout.createParallelGroup()
                .add(jTextField_CustomerID)
                .add(jButton_RequestData))
            .addContainerGap(10, 10)
                .add(jPanel2Layout.createParallelGroup()
                .add(jLabel_CustomerName)
                .add(jTextField_CustomerName))            
            .addContainerGap(10, 10)
                .add(jPanel2Layout.createParallelGroup()
                .add(jLabel_CustomerBalance)
                .add(jTextField_CustomerBalance))            
            .addContainerGap(10, 10)
            .add(jPanel2Layout.createParallelGroup()
                .add(jButton_Done))));

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        getContentPane().setPreferredSize(new Dimension(260, 320));
         
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                .add(layout.createParallelGroup()            
                    .add(jPanel_AppServer)
                    .add(jPanel_ControlsAndData)));
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                .add(layout.createSequentialGroup()
                    .add(jPanel_AppServer)
                    .addContainerGap(10, 10)
                    .add(jPanel_ControlsAndData)));

        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        setTitle("ClientForSingleAppServ");
        pack();
    }                                             
}