/*	File			DB_Arch_AppServer_for_DB1.java
	Purpose			Application SERVER component for illustration of aspects of 
                                    connectivity, coupling, architecture and transparency
	Author			Richard Anthony	(ar26@gre.ac.uk)
	Date			March 2015
*/
package DB_Arch_AppServer_for_DB1;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.*;
import java.util.Timer;
import java.util.TimerTask;
import javax.swing.*;
    
public class DB_Arch_AppServer_for_DB1 extends javax.swing.JFrame implements ActionListener
{
    private static int DB_TYPE1_PORT = 8002;
    private static int DB_TYPE2_PORT = 8003;
    private static int App_TYPE1_PORT = 8004;
    private static int App_TYPE2_PORT = 8005;
    private static int DATABASE_SIZE = 3;
    private static int MAX_CUST_NAME_LEN = 20;
    
    private enum MessageType {REQUEST_RECORD, SEND_RECORD, CLIENT_DISCONNECT};

    // To implement the equivallent of a C/C++/C# structure, in Java a class is used
    final class Application_Message_PDU
    {
        MessageType eMessageType;   // Size 4   Offset 0
        int iCust_ID;               // Size 4   Offset 4
        Boolean bCustNameValid;     // Size 1   Offset 8
        byte[] cCustName;           // Size 21  Offset 9 - 29
        Boolean bCustBalanceValid;  // Size 1   Offset 30
        // Single byte of padding exists in the byte arraay at this point, to align the following int on a 4-byte boundary
        int iCustBalance;           // Size 32  Offset 32
    };

    private static int Application_Message_Size = 36;
    
    ServerSocket m_ListenSock_AppClient;  // Listen for client connections;
    InetAddress m_Local_Address;
    Socket m_ConnectSock_AppClient;
    InputStream m_InStream_AppClient;
    OutputStream m_OutStream_AppClient;
    InetAddress m_ConnectClient_Address;
    Socket m_ConnectSock_DB_Server;
    InputStream m_InStream_DB_Server;
    OutputStream m_OutStream_DB_Server;
    InetAddress m_ConnectDB_Address;
    Boolean m_bConnected_DB_Server;
    Boolean m_bConnected_App_Client;
    Timer m_Timer;
                 
    public DB_Arch_AppServer_for_DB1()
    {
        initComponents();
        GetLocalIPAddress();
        m_bConnected_DB_Server = false;
        m_bConnected_App_Client = false;
        EnableServer();
        InitialiseControls();
    }

    private void InitialiseControls()    
    {    
        JTextField_DBServer_Port.setEnabled(false);
        JTextField_AppServer_Port.setEnabled(false);
        jButton_ClearLog.setEnabled(true); 
        jButton_Done.setEnabled(true); 
        jTextArea_EventLog.setEnabled(false);
    }

    void GetLocalIPAddress()
    {   
       String sHostAddress = "";
        try
        {
            m_Local_Address = InetAddress.getLocalHost();
            sHostAddress = m_Local_Address.toString(); // Expected format: "Hostname/192.168.100.5"
            int iStartOfDottedIPSubstring = sHostAddress.indexOf('/');
            if(-1 == iStartOfDottedIPSubstring)
            {   // Align to the actual start of the dotted IP address part of the string
                iStartOfDottedIPSubstring = 0; // '/' not found
            }
            else
            {
                iStartOfDottedIPSubstring++; // Skip the '/' charachter
            }
            String sDottedIPAddress = sHostAddress.substring(iStartOfDottedIPSubstring);
            JTextField_DBServer_IPAddress.setText(sDottedIPAddress);
            JTextField_DBServer_Port.setText(Integer.toString(DB_TYPE1_PORT));
            JTextField_AppServer_Port.setText(Integer.toString(App_TYPE1_PORT));
        }
        catch (Exception Ex)
        {
            JOptionPane.showMessageDialog(null,Ex.toString(),"DB_Arch_AppServer_for_DB1",JOptionPane.PLAIN_MESSAGE);
            CloseAndExit();
        }
    }
    
    void Accept()
    {
        try
        {
            m_ConnectSock_AppClient = m_ListenSock_AppClient.accept();
            m_ConnectSock_AppClient.setSoTimeout(100); // 100ms timeout (non-blocking)            }
            m_InStream_AppClient = m_ConnectSock_AppClient.getInputStream();
            m_OutStream_AppClient = m_ConnectSock_AppClient.getOutputStream();
        }
        catch (SocketTimeoutException Ex)
        {   
            return;
        }
        catch (Exception Ex)
        {
            WriteEventLogEntry(Ex.toString());
            return;
        }
        m_bConnected_App_Client = true;
        String sStr = String.format("Client %s Connected", m_ConnectSock_AppClient.getRemoteSocketAddress().toString());
        WriteEventLogEntry(sStr);
    }

    void EnableServer()
    {
        try
        {   // Create, Bind and Listen on Listen Socket and set it NonBlocking
            m_ListenSock_AppClient = new ServerSocket(App_TYPE1_PORT, 5/*backlog*/, m_Local_Address/*bindAddr*/);
            m_ListenSock_AppClient.setSoTimeout(100); // Timeout = 100mS i.e. non-blocking
        }
        catch (Exception Ex)  // Catch exceptions from the socket operations
        {
           JOptionPane.showMessageDialog(null,Ex.toString(),"DB_Arch_AppServer_for_DB1",0);
           CloseAndExit();
        }
        Start_Timer();   // As all activity is non-blocking
    }

    private void Start_Timer()
    {
        m_Timer = new Timer();
        m_Timer.scheduleAtFixedRate(
                new OnTimedEvent_Timer(), 100, 100); // Short interval to allow UI responsiveness)
    }

    private class OnTimedEvent_Timer extends TimerTask
    {
        public void run()
        {
            if(false == m_bConnected_App_Client)
            {
                Accept(); // This only applies to the App Client connection
            }
            DoReceive(); // This applies to both connections
        }
    }
    
    void Send_To_AppClient(Application_Message_PDU Message)
    {
        byte[] bBuffer = Serialize_Application_Message_PDU(Message);
        try
        {
            m_OutStream_AppClient.write(bBuffer);
        }
        catch (SocketTimeoutException Ex)
        {
        }
        catch (Exception ex)
        {
        }
    }

    void Send_To_DB_Server(Application_Message_PDU Message)
    {
        byte[] bBuffer = Serialize_Application_Message_PDU(Message);
        try
        {
            m_OutStream_DB_Server.write(bBuffer);
        }
        catch (SocketTimeoutException Ex)
        {
        }
        catch (Exception ex)
        {
        }
    }
 				
    void Forward_DataRow_Message_To_AppClient(Application_Message_PDU Message_DB_In)
    {
        Application_Message_PDU Message_App_Out = new Application_Message_PDU();
        Message_App_Out.cCustName = new byte[MAX_CUST_NAME_LEN + 1];
        switch(Message_DB_In.eMessageType)
        {
            case SEND_RECORD:
                Message_App_Out.eMessageType = MessageType.SEND_RECORD;
                Message_App_Out.iCust_ID = Message_DB_In.iCust_ID;
                Message_App_Out.bCustBalanceValid = Message_DB_In.bCustBalanceValid;
                Message_App_Out.bCustNameValid = Message_DB_In.bCustNameValid;
                Message_App_Out.iCustBalance = 0;
                Message_App_Out.cCustName = new byte[MAX_CUST_NAME_LEN+1];
                
                if(true == Message_DB_In.bCustBalanceValid)
                {	
                    Message_App_Out.iCustBalance = Message_DB_In.iCustBalance;
                }

                if(true == Message_DB_In.bCustNameValid)
                {	
                    System.arraycopy(Message_DB_In.cCustName/*src*/, 0, Message_App_Out.cCustName/*dest*/, 0, Message_DB_In.cCustName.length/*count*/);
                }

                Send_To_AppClient(Message_App_Out);		
                String sStr = String.format("Data Record %d Forwarded to App Client", Message_App_Out.iCust_ID);
                WriteEventLogEntry(sStr);
                break;
        }
    }

    void Forward_Request_Message_To_DB_Server(Application_Message_PDU Message_App_In)
    {
        // This method hides details of the spceific database request format from the client
        // The client makes its request in an application-defined manner, independent of the format used by the database
        Application_Message_PDU Message_DB_Out = new Application_Message_PDU();
        Message_DB_Out.cCustName = new byte[MAX_CUST_NAME_LEN + 1];
        switch(Message_App_In.eMessageType)
        {
            case REQUEST_RECORD:
                Message_DB_Out.eMessageType = MessageType.REQUEST_RECORD;
                Message_DB_Out.iCust_ID = Message_App_In.iCust_ID;
                Message_DB_Out.bCustBalanceValid = false;
                Message_DB_Out.bCustNameValid  = false; 
                Message_DB_Out.iCustBalance = 0;
                Message_DB_Out.cCustName = new byte[MAX_CUST_NAME_LEN+1];
                
                Send_To_DB_Server(Message_DB_Out);		
                String sStr = String.format("Data Record %d request forwarded to DB", Message_DB_Out.iCust_ID);
                WriteEventLogEntry(sStr);			
                break;
        }
    }
    
    void DoReceive()
    {		
        byte[] bArray = new byte[Application_Message_Size];
        int iReceiveByteCount = 0;

        // Check for Application messages (from DB server)
        if(true == m_bConnected_DB_Server)
        {
            iReceiveByteCount = 0;
            try
            {
                iReceiveByteCount = m_InStream_DB_Server.read(bArray);
            }
            catch (SocketTimeoutException Ex)
            {
            }
            catch (Exception Ex)
            {
            }
            if(Application_Message_Size <= iReceiveByteCount)
            {
                Application_Message_PDU Message = DeSerialize_ApplicationMessage(bArray);
                switch(Message.eMessageType)
                {
                    case SEND_RECORD:
                        String sStr = String.format("Data Record %d received", Message.iCust_ID);
                        WriteEventLogEntry(sStr);
                        if(true == m_bConnected_App_Client)
                        {
                            Forward_DataRow_Message_To_AppClient(Message);
                        }
                    break;
                }
            }
        }
        // Check for Application messages (from App client)
        if(true == m_bConnected_App_Client)
        {
            iReceiveByteCount = 0;
            try
            {
                iReceiveByteCount = m_InStream_AppClient.read(bArray);
            }
            catch (SocketTimeoutException Ex)
            {
                return;
            }
            catch (Exception Ex)
            {
                return;
            }
            if(Application_Message_Size <= iReceiveByteCount)
            {
                Application_Message_PDU Message = DeSerialize_ApplicationMessage(bArray);
                switch(Message.eMessageType)
                {
                   case REQUEST_RECORD:
                        String sStr = String.format("Data record %d requested", Message.iCust_ID);
                        WriteEventLogEntry(sStr);
                        if(true == m_bConnected_DB_Server)
                        {
                            Forward_Request_Message_To_DB_Server(Message);
                        }
                        break;		

                    case CLIENT_DISCONNECT:
                        m_bConnected_App_Client = false;
                        CloseClientSocket();
                        WriteEventLogEntry("Client disconnected");
                        break;
                }
            }
        }
    }
    
    void ConnectToDBServer()
    {
        if(true == GetConnectAddress_DB1())
        {
            try
            {
                m_ConnectSock_DB_Server = new Socket(m_ConnectDB_Address, DB_TYPE1_PORT); // Create Socket and connect to Server
                m_InStream_DB_Server = m_ConnectSock_DB_Server.getInputStream();
                m_OutStream_DB_Server = m_ConnectSock_DB_Server.getOutputStream();
                m_ConnectSock_DB_Server.setSoTimeout(100); // Timeout = 100mS i.e. non-blocking
            }  
            catch (Exception Ex)  // Catch exceptions from the socket operations
            {
               JOptionPane.showMessageDialog(null,Ex.toString(),
                   "Is server address/port correct? Is server running?",
                   JOptionPane.PLAIN_MESSAGE);
               return;
            }
            m_bConnected_DB_Server = true;
            jButton_ConnectToDBServer.setEnabled(false);
            JTextField_DBServer_IPAddress.setEnabled(false);
            WriteEventLogEntry("Connected to DB server");
        }
    }
    
    Boolean GetConnectAddress_DB1()
    {
        // Get the server address fron the user interface. Our own address is initially placed there to
        // reduce user interaction requirements (for example if both processes are on the same computer
        // no change to the address is required, or if both computers in the same Class C IP subnet
        // only the fourth address byte needs to be changed). The port number is also initialised on
        // the user interface to a default value, but must be read-in because the use may have changed it
        byte[] bArray = DottedIPAddessString_To_ByteArraySize4(JTextField_DBServer_IPAddress.getText());
        if(0 == bArray[0])
        {
            JOptionPane.showMessageDialog(null,"Mal-formed dotted IP address (use format n.n.n.n)","DB Arch AppClient",JOptionPane.PLAIN_MESSAGE);
            return false;
        }
        try
        {
            m_ConnectDB_Address = InetAddress.getByAddress(bArray);
        }
        catch (Exception Ex)
        {
            JOptionPane.showMessageDialog(null,Ex.toString(),"DB Arch AppClient",JOptionPane.PLAIN_MESSAGE);
            return false;
        }
        return true;
    }

    Application_Message_PDU DeSerialize_ApplicationMessage(byte[] bRecdByteArray)
    {   // Converts a byte array (from Receive) to an Application_Message_PDU structure
        Application_Message_PDU result_Message = new Application_Message_PDU();
        result_Message.eMessageType = MessageType.values()[bRecdByteArray[0]]; // Convert byte value to enumerated MessageType
        byte[] bInt = new byte[4]; 
        System.arraycopy(bRecdByteArray/*src*/, 4, bInt/*dest*/, 0, 4/*count*/); 
        result_Message.iCust_ID = ByteArraySize4_ToInt(bInt);
        result_Message.bCustNameValid = true;
        if (0 == bRecdByteArray[8])
        {
            result_Message.bCustNameValid = false;
        }
        result_Message.cCustName = new byte[MAX_CUST_NAME_LEN+1];
        System.arraycopy(bRecdByteArray/*src*/, 9, result_Message.cCustName/*dest*/, 0, MAX_CUST_NAME_LEN/*count*/); 
        result_Message.bCustBalanceValid = true;
        if (0 == bRecdByteArray[30])
        {
            result_Message.bCustBalanceValid = false;
        }
        System.arraycopy(bRecdByteArray/*src*/, 32, bInt/*dest*/, 0, 4/*count*/); 
        result_Message.iCustBalance = ByteArraySize4_ToInt(bInt);
        return result_Message;
    }

    byte[] Serialize_Application_Message_PDU(Application_Message_PDU Message)
    {   // Convert an Application_Message_PDU structure into a byte array
        byte[] bByteArray = new byte[Application_Message_Size]; // Create a byte array to hold the serialised form of the message PDU
        bByteArray[0] = (byte)Message.eMessageType.ordinal();
        byte[] bInt = IntToByteArray(Message.iCust_ID);
        System.arraycopy(bInt/*src*/, 0, bByteArray/*dest*/, 4, 4/*count*/);
        bByteArray[8] = 0; // false (bCustNameValid)
        if(true == Message.bCustNameValid)
        {
            bByteArray[8] = 1; // true (bCustNameValid)
        }
        if(null != Message.cCustName)
        {
            System.arraycopy(Message.cCustName/*src*/, 0, bByteArray/*dest*/, 9, Message.cCustName.length/*count*/);
        }
        bByteArray[30] = 0; // false (bCustBalanceValid)
        if (true == Message.bCustBalanceValid)
        {
            bByteArray[30] = 1; // true (bCustBalanceValid)
        }
        bInt = IntToByteArray(Message.iCustBalance);
        System.arraycopy(bInt/*src*/, 0, bByteArray/*dest*/, 32, 4/*count*/);
        return bByteArray;
    }

    void CloseClientSocket()
    {
        try
        {
            m_ConnectSock_AppClient.close();
        }
        catch (Exception Ex)
        {   // Generic approach to dealing with situations such as socket not connected
        }
    }
    
    void CloseSockets()
    {
        CloseClientSocket();
        try
        {
            m_ConnectSock_DB_Server.close();
        }
        catch (Exception Ex)
        {   // Generic approach to dealing with situations such as socket not connected
        }
        try
        {
            m_ListenSock_AppClient.close();
        }
        catch (Exception Ex)
        {   // Generic approach to dealing with situations such as socket not connected
        }
    }

    private void CloseAndExit()
    {   // Close all sockets and exit the application
       if(null != m_Timer)
        {
            m_Timer.cancel();
        }
        CloseSockets();
        System.exit(0);
    }

    private void Done_Button_Action()
    {
        if (true == m_bConnected_DB_Server)
        {
            Application_Message_PDU Message_DB_Out = new Application_Message_PDU();
            Message_DB_Out.eMessageType = MessageType.CLIENT_DISCONNECT;// The App Server is a 'client' as seen by the DB Server
            Message_DB_Out.iCust_ID = 0;
            Message_DB_Out.bCustBalanceValid = false;
            Message_DB_Out.bCustNameValid  = false; 
            Message_DB_Out.iCustBalance = 0;
            Message_DB_Out.cCustName = new byte[MAX_CUST_NAME_LEN+1];
            Send_To_DB_Server(Message_DB_Out);
        }
        CloseAndExit();
    }
    
    private byte[] DottedIPAddessString_To_ByteArraySize4(String sDottedIPAddress)
    {
        byte[] bAddress = new byte[4]; // Each byte initialised as value 0
        int iLength = sDottedIPAddress.length();
        if(7 > iLength || 15 < iLength) // A well formed dotted IP address string must be in this length range
        {
            return bAddress; // All 0's signals mal-formed dotted IPaddress string
        }
        // sDottedIPAddress is a '.' separated, list of numeric values
        // There is no '.' after the last item in the list
        int iNextDotPos  = 0;
        String sCurrentValue = "";
        int iArrayIndex = 0;
        try
        {
            while ((iNextDotPos = sDottedIPAddress.indexOf('.')) > 0 && iNextDotPos < iLength) // Ignore chars beyond the valid string content
            {
                sCurrentValue = sDottedIPAddress.substring(0/*start*/, iNextDotPos/*length*/);
                bAddress[iArrayIndex++] = (byte)Integer.parseInt(sCurrentValue);
                sDottedIPAddress = sDottedIPAddress.substring(iNextDotPos + 1/*start*/);// Remove 'dealt with' name from head of list
                iLength -= (iNextDotPos + 1);                                 // Keep track of remaining list-string length
            }
            // Deal with residual substring (there is no comma after the last item in the list)
            bAddress[iArrayIndex++] = (byte)Integer.parseInt(sDottedIPAddress);
        }
        catch (Exception Ex)
        {   // Catch various string-format related exceptions (such as having more than three '.'s which
            // would cause the byte array index to go out of bounds)
            bAddress[0] = 0; // signal mal-formed dotted IPaddress string 
        }
        return bAddress;
    }
    
    private static byte[] IntToByteArray(int iVal)
    {
        byte[] bArray = new byte[4];
        bArray[0] = (byte)(iVal & 0x000000ff);
        bArray[1] = (byte)((iVal & 0x0000ff00) >> 8);
        bArray[2] = (byte)((iVal & 0x00ff0000) >> 16);
        bArray[3] = (byte)((iVal & 0xff000000) >> 24);
        return bArray;
    }

    private int ByteArraySize4_ToInt(byte[] bVal)
    {
        if(4 != bVal.length)
        {
            return -1;  // Signal invalid byte array length
        }
        int iResult = bVal[0] & 0xFF;
        iResult += ((bVal[1] & 0xFF) << 8);
        iResult += ((bVal[2] & 0xFF) << 16);
        iResult += ((bVal[3] & 0xFF) << 24);            
        return iResult;
    }

    private String TruncateString(String sStr, int iMaxLength)
    {
        try
        {
            // Truncate to null terminator if string contains null-termiation chars (happens when converting from a byte array)
            int iNullTerminatePosition = sStr.indexOf(0);
            if(-1 != iNullTerminatePosition)
            {
                sStr = sStr.substring(0, iNullTerminatePosition);
            }
            if(sStr.length()> iMaxLength)
            {   // Only use substring if the string is actually longer than the second parameter
                // (the length limit) otherwise an ArgumentOutOfRangeException is thrown
                sStr = sStr.substring(0, iMaxLength);
            }
            return sStr;
        }
        catch (Exception Ex)
        {
            return "";
        }
    }
   
    private void WriteEventLogEntry(String sStr)
    {
        sStr += "\n";
        jTextArea_EventLog.append(sStr);       
    }

    public static void main(String args[]) throws Exception
    {
        // Initialise the GUI libraries
        try 
        {
            javax.swing.UIManager.LookAndFeelInfo[] installedLookAndFeels=javax.swing.UIManager.getInstalledLookAndFeels();
            for (int idx=0; idx<installedLookAndFeels.length; idx++)
            {
                if ("Nimbus".equals(installedLookAndFeels[idx].getName())) {
                    javax.swing.UIManager.setLookAndFeel(installedLookAndFeels[idx].getClassName());
                    break;
                }
            }
        } 
        catch (Exception Ex)
        {
            System.exit(0);
        }

        // Create and display the GUI form
        java.awt.EventQueue.invokeAndWait(new Runnable()
        { 
            public void run() 
            { 
                new DB_Arch_AppServer_for_DB1().setVisible(true);
            }
        } );
    }    

    public void actionPerformed(ActionEvent e)
    {
        if(jButton_ConnectToDBServer == e.getSource())
        {
            ConnectToDBServer();
        }
        if(jButton_ClearLog == e.getSource())
        {
            jTextArea_EventLog.setText("");
        }
        if(jButton_Done == e.getSource())
        {
            Done_Button_Action();
        }
    }
    
    private JPanel jPanel_CommunicationConfiguration;
    private JLabel jLabel_DBServer_IPAddress;
    private JTextField JTextField_DBServer_IPAddress;
    private JLabel jLabel_DBServer_Port1;
    private JLabel jLabel_DBServer_Port2;
    private JTextField JTextField_DBServer_Port;
    private JLabel jLabel_AppServer_Port1;
    private JLabel jLabel_AppServer_Port2;
    private JTextField JTextField_AppServer_Port;
    private JPanel jPanel_Controls;
    private JButton jButton_ConnectToDBServer;
    private JButton jButton_ClearLog;
    private JButton jButton_Done;
    private JPanel jPanel_EventLog;
    private JTextArea jTextArea_EventLog;
    private JScrollPane jScrollPane_EventLog;

    private void initComponents()
    {
        jLabel_DBServer_IPAddress = new JLabel();
        jLabel_DBServer_IPAddress.setText("DB Server IPAddress");
        JTextField_DBServer_IPAddress = new JTextField();
        JTextField_DBServer_IPAddress.setMaximumSize(new Dimension(110, 20));
        JTextField_DBServer_IPAddress.setHorizontalAlignment(JTextField.CENTER);
        jLabel_DBServer_Port1 = new JLabel();
        jLabel_DBServer_Port1.setText("DB Server Port (which this");
        jLabel_DBServer_Port2 = new JLabel();
        jLabel_DBServer_Port2.setText("application server connects to)");
        JTextField_DBServer_Port = new JTextField();
        JTextField_DBServer_Port.setMaximumSize(new Dimension(50, 20));
        JTextField_DBServer_Port.setHorizontalAlignment(JTextField.CENTER);
        jLabel_AppServer_Port1 = new JLabel();
        jLabel_AppServer_Port1.setText("Application Server Port (for the");
        jLabel_AppServer_Port2 = new JLabel();
        jLabel_AppServer_Port2.setText("application client to connect to)");
        JTextField_AppServer_Port = new JTextField();
        JTextField_AppServer_Port.setMaximumSize(new Dimension(50, 20));
        JTextField_AppServer_Port.setHorizontalAlignment(JTextField.CENTER);
        jPanel_CommunicationConfiguration = new JPanel();
        jPanel_CommunicationConfiguration.setPreferredSize(new Dimension(320, 200));
        jPanel_CommunicationConfiguration.setBorder(BorderFactory.createTitledBorder("Communication configuration"));

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel_CommunicationConfiguration);
        jPanel_CommunicationConfiguration.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(jLabel_DBServer_IPAddress)
                        .addContainerGap(10, 10)
                        .add(JTextField_DBServer_IPAddress))
                     .add(jPanel1Layout.createSequentialGroup()
                        .add(jPanel1Layout.createParallelGroup()
                            .add(jLabel_DBServer_Port1)
                            .add(jLabel_DBServer_Port2))
                        .addContainerGap(10, 10)
                        .add(JTextField_DBServer_Port))
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(jPanel1Layout.createParallelGroup()
                            .add(jLabel_AppServer_Port1)
                            .add(jLabel_AppServer_Port2))
                        .addContainerGap(10, 10)
                        .add(JTextField_AppServer_Port))));
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                .add(jPanel1Layout.createSequentialGroup()
                    .add(jPanel1Layout.createParallelGroup()
                        .add(jLabel_DBServer_IPAddress)
                        .add(JTextField_DBServer_IPAddress))
                    .addContainerGap(20, 20)
                    .add(jPanel1Layout.createParallelGroup()
                        .add(jPanel1Layout.createSequentialGroup()
                            .add(jLabel_DBServer_Port1)
                            .add(jLabel_DBServer_Port2))
                        .add(JTextField_DBServer_Port))
                    .addContainerGap(20, 20)
                    .add(jPanel1Layout.createParallelGroup()
                        .add(jPanel1Layout.createSequentialGroup()
                            .add(jLabel_AppServer_Port1)
                            .add(jLabel_AppServer_Port2))
                        .add(JTextField_AppServer_Port))));

        jButton_ConnectToDBServer = new javax.swing.JButton();
        jButton_ConnectToDBServer.setText("Connect to DB Server");
        jButton_ConnectToDBServer.setMaximumSize(new Dimension(30, 4));
        jButton_ConnectToDBServer.addActionListener(this);
        jButton_ClearLog = new javax.swing.JButton();
        jButton_ClearLog.setText("Clear Log entries");
        jButton_ClearLog.setMaximumSize(new Dimension(30, 4));
        jButton_ClearLog.addActionListener(this);
        jButton_Done = new javax.swing.JButton();
        jButton_Done.setText("Done");
        jButton_Done.setMaximumSize(new Dimension(30, 4));
        jButton_Done.addActionListener(this);
        jPanel_Controls = new javax.swing.JPanel();
        jPanel_Controls.setPreferredSize(new Dimension(500, 200));
        jPanel_Controls.setBorder(javax.swing.BorderFactory.createTitledBorder(""));

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel_Controls);
        jPanel_Controls.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                .add(jButton_ConnectToDBServer)
                .add(jButton_ClearLog)
                .add(jButton_Done));
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                .add(jPanel2Layout.createSequentialGroup()
                    .add(jButton_ConnectToDBServer)
                    .addContainerGap(10, 10)
                    .add(jButton_ClearLog)
                    .addContainerGap(10, 10)
                    .add(jButton_Done)));
        jPanel2Layout.linkSize(new java.awt.Component[] {jButton_ConnectToDBServer, jButton_ClearLog, jButton_Done});

        jTextArea_EventLog = new javax.swing.JTextArea();
        jScrollPane_EventLog = new javax.swing.JScrollPane(jTextArea_EventLog,
                ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, 
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        jPanel_EventLog = new javax.swing.JPanel();
        jPanel_EventLog.setPreferredSize(new Dimension(280, 330));
        jPanel_EventLog.setBorder(javax.swing.BorderFactory.createTitledBorder("Event Log (Diagnostic)"));
        
        org.jdesktop.layout.GroupLayout jPanel3Layout = new org.jdesktop.layout.GroupLayout(jPanel_EventLog);
        jPanel_EventLog.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
            .add(jScrollPane_EventLog));
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
            .add(jScrollPane_EventLog));        
         
        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        getContentPane().setPreferredSize(new Dimension(560, 330));
         
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)            
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)  
                            .add(jPanel_CommunicationConfiguration)
                            .add(jPanel_Controls))
                        .add(layout.createParallelGroup()  
                            .add(jPanel_EventLog)))));
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.CENTER)
                .add(layout.createParallelGroup()  
                    .add(layout.createSequentialGroup()
                        .add(jPanel_CommunicationConfiguration)
                        .addContainerGap(30, 30)
                        .add(jPanel_Controls)
                        .addContainerGap(30, 30))
                    .add(layout.createSequentialGroup()
                        .add(jPanel_EventLog))));

        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        setTitle("DB Arch AppServer - Connects to DB Server type 1");
        pack();
    }                                             
}