/*	File			KnownPeers.java
	Purpose			Refactoring demonstration
	Author			Richard Anthony	(ar26@gre.ac.uk)
	Date			March 2015
*/

package MediaShare_Peer_Disconnect;

import java.net.InetAddress;
import java.util.Arrays;

public class KnownPeers {
    public static int MAX_KNOWN_PEERS = 10; // Demonstration, therefore a fixed array of ten peers is adequate

    public final class Known_Peer_details 
    {
        int iPeer_ID;
        byte[] bAddress;
        Boolean bInUse;
        Boolean bMediaDataReceived;
        int iMostRecentHeartbeat_TimeDetected;
    };
    
    private Known_Peer_details[] m_Known_Peers_List;
    private MediaShare_Peer_Disconnect m_Parent;

    public KnownPeers(MediaShare_Peer_Disconnect Parent)
    {
        m_Parent = Parent;  // Keep a reference to the parent object, so callbacks can be made, see
        // code in Add_NewlyDiscoveredPeerTo_KnownPeersList() and CheckPeerHeartbeatStatus() methods

        Initialise_KnownPeers_Array();
    }
 
    private void Initialise_KnownPeers_Array()
    {
        m_Known_Peers_List = new Known_Peer_details[MAX_KNOWN_PEERS];
        for (int iIndex = 0; iIndex < MAX_KNOWN_PEERS; iIndex++)
        {
            m_Known_Peers_List[iIndex] = new Known_Peer_details();
            m_Known_Peers_List[iIndex].bAddress = new byte[4];            
            m_Known_Peers_List[iIndex].iPeer_ID = -1;
            m_Known_Peers_List[iIndex].bInUse = false;		
            m_Known_Peers_List[iIndex].bMediaDataReceived = false;
        }
    }

    private int GetIndexOf_Available_KnownPeersList_Entry()
    { 
        int iNextAvailableEntryIndex = -1;	// Signals that the array is full
        for(int iIndex = 0; iIndex < MAX_KNOWN_PEERS; iIndex++)
        {
            if(false == m_Known_Peers_List[iIndex].bInUse)
            {
                iNextAvailableEntryIndex = iIndex;
                break;
            }
        }
        return iNextAvailableEntryIndex;
    }

    public int GetIndexOf_KnownPeer_InPeersList(int iPeerID)
    {	
        for(int iIndex = 0; iIndex < MAX_KNOWN_PEERS; iIndex++)
        {
            if(true == m_Known_Peers_List[iIndex].bInUse)
            {
                if(iPeerID == m_Known_Peers_List[iIndex].iPeer_ID)
                {  // Matched by Peer ID
                    return iIndex;
                }
            }
        }
        return -1;
    }

    public Boolean CheckIfPeerExistsIn_KnownPeersList(int iPeerID, byte[] bAddress)
    { 
        for(int iIndex = 0; iIndex < MAX_KNOWN_PEERS; iIndex++)
        {
            if(true == m_Known_Peers_List[iIndex].bInUse)
            {
                if(iPeerID == m_Known_Peers_List[iIndex].iPeer_ID &&
                    true == Arrays.equals(m_Known_Peers_List[iIndex].bAddress, bAddress))
                {  // Matched by combination of ID and IP address
                    return true;
                }
            }
        }
        return false;
    }

    public Boolean Add_NewlyDiscoveredPeerTo_KnownPeersList(int iPeer_ID, byte[] bAddress)
    {
        int iIndex = GetIndexOf_Available_KnownPeersList_Entry();
        if(-1 == iIndex)
        {
            return false;	// No space in array
        }		
        m_Known_Peers_List[iIndex].bInUse = true;
        m_Known_Peers_List[iIndex].iPeer_ID = iPeer_ID;
        System.arraycopy(bAddress/*src*/, 0, m_Known_Peers_List[iIndex].bAddress/*dest*/, 0, 4/*count*/);
        m_Known_Peers_List[iIndex].iMostRecentHeartbeat_TimeDetected = m_Parent.GetHeartbeatClock();        
        return true;
    }
	
    public Known_Peer_details get_Known_Peers_List_entry_byIndex(int iIndex)
    {
        if(iIndex >= 0 && iIndex < MAX_KNOWN_PEERS)
        {
                return m_Known_Peers_List[iIndex];
        }

        // Initialise an empty structure to return if parameter is out of range
        Known_Peer_details empty_struct = new Known_Peer_details(); 
        empty_struct.bInUse = false;
        empty_struct.bMediaDataReceived = false;
        empty_struct.iPeer_ID = -1;
        empty_struct.bAddress = InetAddress.getLoopbackAddress().getAddress();
        return empty_struct;
    }	

    public void set_Known_Peers_List_entry_bMediaDataReceived_flag_byIndex(int iIndex)
    {
        if(iIndex >= 0 && iIndex < MAX_KNOWN_PEERS)
        {
                m_Known_Peers_List[iIndex].bMediaDataReceived = true;
        }
    }   

    public void set_Known_Peers_HeartbeatStatus_byIndex(int iIndex, int m_iHeartbeatClock)
    {
            if(iIndex >= 0 && iIndex < MAX_KNOWN_PEERS)
            {
                    m_Known_Peers_List[iIndex].iMostRecentHeartbeat_TimeDetected = m_iHeartbeatClock;
            }
    }

    public void CheckPeerHeartbeatStatus(int iCurrentHeartbeatClock)
    {	// Check  Heartbeat Status for all known peers	
        int iEffective_CurrentHeartbeatClock; // Takes care of clock rollovers

        for(int iIndex = 0; iIndex < MAX_KNOWN_PEERS; iIndex++)
        {
            if(true == m_Known_Peers_List[iIndex].bInUse)
            {
                // Heartbeat count increments every 5 seconds, up to a maximum of HeartbeatClock_LIMIT increments (modulo)
                // Heartbeat count rolls over when it reaches HeartbeatClock_LIMIT, back to 0 (%HeartbeatClock_LIMIT)
                if(iCurrentHeartbeatClock < m_Known_Peers_List[iIndex].iMostRecentHeartbeat_TimeDetected)
                {   // Current clock value is lower, assume it has rolled over, so treat it as having a value of +HeartbeatClock_LIMIT
                    // for the purposes of the calculations (so the current clock value is always higher than the peer's stored value
                    // in iMostRecentHeartbeat_TimeDetected).
                    iEffective_CurrentHeartbeatClock = iCurrentHeartbeatClock + MediaShare_Peer_Disconnect.HeartbeatClock_LIMIT;
                }
                else
                {   // Use Current clock value as is.
                    iEffective_CurrentHeartbeatClock = iCurrentHeartbeatClock;
                }

                if ((m_Known_Peers_List[iIndex].iMostRecentHeartbeat_TimeDetected + MediaShare_Peer_Disconnect.PeerDeparted_Threshold_HeartbeatClockInterval)
                            <= iEffective_CurrentHeartbeatClock)
                {   // Peer's iMostRecentHeartbeat_TimeDetected value is out-of-date, Peer is considered to have departed
                    m_Known_Peers_List[iIndex].bInUse = false;
                    m_Known_Peers_List[iIndex].bMediaDataReceived = false;				
                    m_Parent.Callback_PeerDeparted_UpdateStatus(m_Known_Peers_List[iIndex].iPeer_ID);
                }
            }
        }
    }
}